#       zp_parseglobs.bash
#       This file is part of zenpakt

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

$$

zp_parseglobs() {
  #accepts extglob patterns enclosed in an outer
  #all globs should be within one extended glob construct, first extglob syntax from left is taken as glob and rest is evaluated as non glob
  #returns an indexed array zp_glob:
  # 0: nonglob portion before the actual glob
  # 1: glob portion (should be used non quoted)
  # 2: non glob portion after the first glob
  #check/parse the input
  # return status:
  # 1: :syntax error, unexpected ending ( looking for  ')' )
  # if glob not found, zp_glob[0]=$1 and ${#zp_glob[@]}=1
  local nonglob1 nonglob2 glob
  zp_glob=()

  [[ $1 = *@(+|"*"|"!"|@)\(* ]] || { zp_glob=( "$1" ) ; return 0 ;}

  nonglob1=${1%%@(+|"*"|"!"|@)\(*}
  set -- "${1#$nonglob1}"
  level=1
  i=2

  while  ((i<${#1})); do

    if [[ "${1:i:1}" = ")" ]] ; then

        if ((level > 0 )) ; then
          ((level--))
        fi


    elif [[ "${1:i:1}" = \\ ]] ; then

        ((i++))

    elif [[ "${1:i:1}" =  @("@"|"*"|"!"|"?") ]] ; then
        if [[ ${1: ++i:1} = "(" ]];then
         ((level++))
        else
          ((i--))
        fi


    #else
     #echo "${1:i:1}"

    fi

    ((level == 0)) && break
    ((i++))

  done

  ((level == 0 )) || return 1

  #break and quote
  glob=${1:0: ++i}
  nonglob2=${1:i}
  zp_glob=( "$nonglob1" "$glob" "$nonglob2" )
}
